package swarm

import (
	"fmt"
	"github.com/g8rswimmer/go-twitter/v2"
	"github.com/xueqianLu/twitter-bee/client"
	"golang.org/x/time/rate"
	"sync"
	"time"
)

type ClientWithRateLimiter struct {
	*client.BeeClient
	RateLimit *rate.Limiter
}

type Swarm struct {
	clients map[string]*ClientWithRateLimiter
	mu      sync.Mutex
}

var (
	gSwarm *Swarm
)

func GetSwarm() *Swarm {
	if gSwarm == nil {
		gSwarm, _ = InitSwarm(nil)
	}
	return gSwarm
}

func InitSwarm(initialBees []string) (*Swarm, error) {
	s := &Swarm{
		clients: make(map[string]*ClientWithRateLimiter),
	}
	for _, bee := range initialBees {
		s.AddClient(bee)
	}
	return s, nil
}

func (s *Swarm) AddClient(url string) {
	s.mu.Lock()
	defer s.mu.Unlock()
	cli := new(ClientWithRateLimiter)
	cli.BeeClient = client.NewBeeClient(url)
	cli.RateLimit = rate.NewLimiter(rate.Every(15*time.Minute), 40)

	s.clients[url] = cli
}

func (s *Swarm) RemoveClient(url string) {
	s.mu.Lock()
	defer s.mu.Unlock()
	delete(s.clients, url)
}

func (s *Swarm) GetFollowerCount(userID string) (int, error) {
	s.mu.Lock()
	defer s.mu.Unlock()

	for _, cli := range s.clients {
		res, err := cli.GetFollowerCount(userID)
		if err == nil {
			return res.Count, nil
		}
	}
	return 0, fmt.Errorf("can not get the %v follower count", userID)
}

func (s *Swarm) GetFollowerList(user string, cursor string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {
	s.mu.Lock()
	defer s.mu.Unlock()

	for _, cli := range s.clients {
		if cli.RateLimit.Allow() == false {
			continue
		}
		res, err := cli.GetFollowerList(user, cursor)
		if err == nil {
			list := make([]*twitter.UserObj, 0, len(res.List))
			for _, u := range res.List {
				list = append(list, &twitter.UserObj{
					ID:       u.ID,
					Name:     u.Name,
					UserName: u.UserName,
				})
			}
			return list, res.Next, nil, nil
		} else {

		}
	}
	return nil, "", nil, fmt.Errorf("can not get the %v follower list", user)
}
